# -*- mode:python; coding:utf-8 -*-

# Copyright (c) 2020 IBM Corp. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
trestle - A python library and command line utility for compliance.

Trestle is a tool to which enables the creation and validation of
documentation artifacts for compliance requirements. It leverages NIST's
OSCAL (https://pages.nist.gov/OSCAL/documentation/) as a standard data
format for interchange between tools & people and provides an
opinionated approach to OSCAL adoption.
"""

__version__ = '0.2.1'
